﻿#pragma TextEncoding="UTF-8"
#pragma rtGlobals=3
#pragma ModuleName=QuickScatterPlot
#pragma version=1.20
#pragma IgorVersion=8
#include <Resize Controls>

// Quick Scatter Plot

// For efficiently exploring relationships among 1D waves.
// Provides an interface to generate and preview Y vs X scatter plots.
// Allows rapid selection of different pairs of waves to look for correlations.

// This code does not help to produce beautiful scatter plots. Igor has
// built-in and extensively customizable ways to create 2D and 3D scatter
// plots, as well as scatter plot matrices. Rather, this is a tool for
// quickly and visually exploring the degree of correlation between
// different quantities.

// headers used by the updater project
static constant kProjectID=21391 // the project node on IgorExchange
static strconstant ksShortTitle="QuickScatterPlot" // the project short title on IgorExchange

menu "New"
	"Quick Scatter Plot...", /Q, QuickScatterPlot#MakeScatterPlotPanel()
end

static function/DF GetPackageDFREF()
	DFREF dfr = root:Packages:QuickScatterPlot
	if (DataFolderRefStatus(dfr) != 1)
		DFREF dfr = CreatePackageData()
	endif
	return dfr
end

static function/DF CreatePackageData()
	// Create the package data folder
	NewDataFolder/O root:Packages
	NewDataFolder/O root:Packages:QuickScatterPlot
	DFREF dfr = root:Packages:QuickScatterPlot
	
	wave w = ListToTextWave(WaveList("*",";","DIMS:1,CMPLX:0,DF:0,TEXT:0,WAVE:0"), ";")
	Duplicate/O w dfr:Xwaves /wave=XWaves
	Duplicate/O w dfr:Ywaves /wave=YWaves
	Make/O/T/N=1 dfr:XTitle /wave=XTitle
	Make/O/T/N=1 dfr:YTitle /wave=YTitle
	XTitle = "X Wave"
	YTitle = "Y Wave(s)"
	Make/O/N=(numpnts(YWaves)) dfr:selWaveY /wave=selWaveY
	selWaveY = 0
	
	return root:Packages:QuickScatterPlot
end

static function MakeScatterPlotPanel()
	
	DoWindow/F ScatterPanel
	if (v_flag)
		return 1
	endif
	
	DFREF dfr = GetPackageDFREF()
	wave/T/SDFR=dfr XWaves, Ywaves, XTitle, YTitle
	wave/SDFR=dfr selWaveY
	
	NewPanel/K=1/N=ScatterPanel/W=(100,100,725,410) as "Quick Scatter Plot"
	
	DefineGuide/W=ScatterPanel guideR={FR,-5}
	DefineGuide/W=ScatterPanel guideDiv={FL,0.5,Fr}
	DefineGuide/W=ScatterPanel guideL={guideDiv,2.5}
	DefineGuide/W=ScatterPanel guideT={FT,5}
	DefineGuide/W=ScatterPanel guideB={FB,-30}
	
	Display/FG=(guideL,guideT,guideR,guideB)/HOST=ScatterPanel/N=PreviewPlot
	ModifyGraph/W=ScatterPanel#PreviewPlot frameStyle=0
	SetActiveSubwindow ScatterPanel
	
	ListBox ListBoxXWaves,pos={5, 5},size={150, 300}, mode=1, listwave=XWaves, titleWave=XTitle, selRow=-1
	ListBox ListBoxXWaves,Proc=QuickScatterPlot#ScatterListBoxProc, focusRing=0
	ListBox ListBoxYWaves,pos={160, 5},size={150, 300}, mode=9, listwave=YWaves, selWave=selWaveY, titleWave=YTitle
	ListBox ListBoxYWaves,Proc=QuickScatterPlot#ScatterListBoxProc, focusRing=0
	
	PopupMenu popStyle,pos={325,285},size={82,20},title="Style", Proc=QuickScatterPlot#PopupProc
	PopupMenu popStyle,mode=1,popvalue="none",value="none;"+MacroList("*", ";", "SUBTYPE:GraphStyle")
	Button btnDoIt,pos={565,285},size={50,20},title="Do It", Proc=QuickScatterPlot#ButtonProc
		
	SetWindow ScatterPanel activeChildFrame=0
	SetWindow ScatterPanel hook(hPointID)=QuickScatterPlot#HookScatterPlot
	
	// resizing user data for controls
	ListBox ListBoxXWaves,win=ScatterPanel,userdata(ResizeControlsInfo)=A"!!,?X!!#9W!!#A%!!#BPz!!#](Aon\"Qzzzzzzzzzzzzzz!!\";f=(u2eBE/#4z"
	ListBox ListBoxXWaves,win=ScatterPanel,userdata(ResizeControlsInfo)+=A"zzzzzzzzzzzz!!#u:Du]k<zzzzzzzzzzz"
	ListBox ListBoxXWaves,win=ScatterPanel,userdata(ResizeControlsInfo)+=A"zzz!!#?(FEDG<!-N!dAO6@szzzzzzzzzzzz!!!"
	ListBox ListBoxYWaves,win=ScatterPanel,userdata(ResizeControlsInfo)=A"!!,G0!!#9W!!#A%!!#BPz!!\";f=(u2eBE/#4zzzzzzzzzzzzz!!#N3Bk1ctB6@p!AOL6U"
	ListBox ListBoxYWaves,win=ScatterPanel,userdata(ResizeControlsInfo)+=A"zzzzzzzzzzzz!!#u:Du]k<zzzzzzzzzzz"
	ListBox ListBoxYWaves,win=ScatterPanel,userdata(ResizeControlsInfo)+=A"zzz!!#?(FEDG<!-N!dAO6@szzzzzzzzzzzz!!!"
	PopupMenu popStyle,win=ScatterPanel,userdata(ResizeControlsInfo)=A"!!,H]J,hrsJ,hq=!!#<Xz!!#N3Bk1ctB6@p!AOL6Uzzzzzzzzzzzz!!#N3Bk1ctB6@p!AOL6U"
	PopupMenu popStyle,win=ScatterPanel,userdata(ResizeControlsInfo)+=A"zzzzzzzzzzzz!!#?(FEDG<zzzzzzzzzzz"
	PopupMenu popStyle,win=ScatterPanel,userdata(ResizeControlsInfo)+=A"zzz!!#?(FEDG<!-N!dAO6@szzzzzzzzzzzz!!!"
	Button btnDoIt,win=ScatterPanel,userdata(ResizeControlsInfo)=A"!!,Is5QF03J,ho,!!#<Xz!!#o2B4uAeB6@p!AOL6Uzzzzzzzzzzzz!!#o2B4uAeB6@p!AOL6U"
	Button btnDoIt,win=ScatterPanel,userdata(ResizeControlsInfo)+=A"zzzzzzzzzzzz!!#?(FEDG<zzzzzzzzzzz"
	Button btnDoIt,win=ScatterPanel,userdata(ResizeControlsInfo)+=A"zzz!!#?(FEDG<!-N!dAO6@szzzzzzzzzzzz!!!"
	
	// resizing user data for panel window
	SetWindow kwTopWin,userdata(ResizeControlsInfo)=A"!!*'\"z!!#D,5QF0@zzzzzzzzzzzzzzzzzzzzz"
	SetWindow kwTopWin,userdata(ResizeControlsInfo)+=A"zzzzzzzzzzzzzzzzzzzzzzzzz"
	SetWindow kwTopWin,userdata(ResizeControlsInfo)+=A"zzzzzzzzzzzzzzzzzzz!!!"
	SetWindow kwTopWin,userdata(ResizeControlsGuides)="guideR;guideDiv;guideL;guideT;guideB;"
	SetWindow kwTopWin,userdata(ResizeControlsInfoguideR)="NAME:guideR;WIN:ScatterPanel;TYPE:User;HORIZONTAL:0;POSITION:620.00;GUIDE1:FR;GUIDE2:;RELPOSITION:-5;"
	SetWindow kwTopWin,userdata(ResizeControlsInfoguideDiv)="NAME:guideDiv;WIN:ScatterPanel;TYPE:User;HORIZONTAL:0;POSITION:312.00;GUIDE1:FL;GUIDE2:Fr;RELPOSITION:0.5;"
	SetWindow kwTopWin,userdata(ResizeControlsInfoguideL)="NAME:guideL;WIN:ScatterPanel;TYPE:User;HORIZONTAL:0;POSITION:314.00;GUIDE1:guideDiv;GUIDE2:;RELPOSITION:2.5;"
	SetWindow kwTopWin,userdata(ResizeControlsInfoguideT)="NAME:guideT;WIN:ScatterPanel;TYPE:User;HORIZONTAL:1;POSITION:5.00;GUIDE1:FT;GUIDE2:;RELPOSITION:5;"
	SetWindow kwTopWin,userdata(ResizeControlsInfoguideB)="NAME:guideB;WIN:ScatterPanel;TYPE:User;HORIZONTAL:1;POSITION:280.00;GUIDE1:FB;GUIDE2:;RELPOSITION:-30;"
	
	// resizing panel hook
	SetWindow ScatterPanel hook(ResizeControls)=ResizeControls#ResizeControlsHook
	
	return 1
end

static function PopupProc(STRUCT WMPopupAction &s) : PopupMenuControl
	if (s.eventCode == 2)
		SetActiveSubwindow ScatterPanel#PreviewPlot
		if (cmpstr(s.popStr, "none"))
			Execute/Z/Q s.popStr + "()"
		else
			UpdateScatterPlot()
		endif
	endif
	return 0
end

static function ButtonProc(STRUCT WMButtonAction &s) : ButtonControl
	if (s.eventcode == 2)
		string cmd = WinRecreation("ScatterPanel#PreviewPlot", 0)
		cmd = ReplaceString("/FG=(guideL,guideT,guideR,guideB)/HOST=#", cmd, "")
		cmd = RemoveListItem(0, cmd, "\r")
		cmd = RemoveListItem(0, cmd, "\r")
		cmd = ReplaceString("EndMacro", cmd, "")
		cmd = ReplaceString("\r", cmd, ";")
		Execute/Z/Q cmd
		KillWindow/Z ScatterPanel
	endif
	return 0
end

static function ScatterListBoxProc(STRUCT WMListboxAction &s)
			
	if (s.eventcode != 4 && s.eventcode != 5)
		return 0
	endif
		
	DFREF dfr=root:Packages:QuickScatterPlot
	
	if (cmpstr(s.ctrlName, "ListBoxXWaves")==0)
		wave selwave = dfr:selWaveY
		wave/T YwaveNames = dfr:YWaves
		string strXwave = ""
		if (s.row>-1 && s.row<numpnts(s.listWave))
			strXwave = s.listWave[s.row]
		endif
		selwave = SelectableStatus(YwaveNames, strXwave, selwave)
	endif
	UpdateScatterPlot()
end

// reset listboxes and plot when list of waves in current data folder changes
static function UpdateListboxes()
	DFREF dfr = GetPackageDFREF()
	wave/T/SDFR=dfr XWaves, Ywaves, XTitle, YTitle
	wave/SDFR=dfr selWaveY
	
	wave/T w = ListToTextWave(WaveList("*",";","DIMS:1,CMPLX:0,DF:0,TEXT:0,WAVE:0"), ";")
	if (TextWavesIdentical(w, XWaves))
		return 0
	endif
	
	ControlInfo/W=ScatterPanel ListBoxXWaves
	variable selX = V_Value
	Duplicate/free/T ywaves oldYwaves
	Duplicate/free selWaveY oldSelWaveY
	
	Duplicate/O/T w dfr:Xwaves /wave=XWaves
	Duplicate/O/T w dfr:Ywaves /wave=YWaves
	Make/O/N=(numpnts(YWaves)) dfr:selWaveY /wave=selWaveY
	selWaveY = 128
	
	int selection = -1
	
	if (selX >= 0)
		FindValue/TEXT=oldYwaves[selX] w
		selection = v_row
	endif
	ListBox ListBoxXWaves win=ScatterPanel, selRow=selection
	
	if (selection > -1)
		selWaveY = SelectableStatus(YWaves, w[selection], selWaveY)
	endif
	
	int i
	for (i=numpnts(oldYwaves)-1;i>=0;i--)
		if (oldSelWaveY[i] & 9)
			FindValue/TEXT=oldYwaves[i] w
			if (v_row > -1)
				selWaveY[v_row] = !(selWaveY[v_row] & 128) ? 1 : selWaveY[v_row]
			endif
		endif
	endfor
	
	UpdateScatterPlot()
end

static function TextWavesIdentical(wave/T w1, wave/T w2)
	if (numpnts(w1) && (numpnts(w1) == numpnts(w2)))
		Make/free/N=(numpnts(w1)) wCmp
		wCmp = cmpstr(w1, w2)
		if (WaveMax(wCmp)==0 && WaveMin(wCmp)==0)
			return 1
		endif
	endif
	return 0
end

static function UpdateScatterPlot()	
	variable numTraces = ItemsInList(TraceNameList("ScatterPanel#PreviewPlot",";",1))
	variable i
	for (i=0;i<numTraces;i+=1)
		RemoveFromGraph/Z/W=ScatterPanel#PreviewPlot $"#0"
	endfor
	
	DFREF dfr = root:Packages:QuickScatterPlot
	wave/SDFR=dfr/T xWaves, yWaves
	wave/SDFR=dfr selWaveY
	ControlInfo/W=ScatterPanel ListBoxXWaves
	if (V_Value == -1)
		return 1
	endif
	wave/Z xWave = $xWaves[V_Value]
	if (WaveExists(xWave) == 0)
		return 0
	endif
	
	// make a list of some solid marker types
	Make/free/B/U markers = {19,16,17,18,23,26,29,46,49,52,55,58,60,62,14,15,32,34,36,38}
	wave colors = ColorWave()
	
	variable traceNum=0, colorIndex, markerIndex
	for (i=0;i<numpnts(YWaves);i++)
		if ( (selWaveY[i]&1) || (selWaveY[i]&8) )
			wave/Z yWave = $YWaves[i]
			if (WaveExists(yWave))
				AppendToGraph/W=ScatterPanel#PreviewPlot yWave vs xWave
				markerIndex = mod(traceNum, DimSize(markers, 0))
				ModifyGraph/W=ScatterPanel#PreviewPlot mode[traceNum]=3,marker[traceNum]=markers[markerIndex],useMrkStrokeRGB[traceNum]=1
				colorIndex = mod(traceNum, DimSize(colors, 0))
				ModifyGraph/W=ScatterPanel#PreviewPlot rgb[traceNum]=(colors[colorIndex][0],colors[colorIndex][1],colors[colorIndex][2])
				traceNum += 1
			endif
		endif
	endfor
	
	ControlInfo/W=ScatterPanel popStyle
	if (cmpstr(S_Value, "none"))
		Execute/Z/Q s_value + "()"
	endif
	
end

static function SelectableStatus(string strYwave, string strXwave, variable sel)
	
	wave/Z xWave = $strXwave
	wave/Z yWave = $strYwave
	
	if (WaveExists(xWave)==0 || WaveExists(yWave)==0)
		return 0x80
	endif
	
	if (numpnts(xWave) == numpnts(yWave))
		return sel - (sel & 0x80)
	else
		return 0x80
	endif
end

static function /wave ColorWave()
	Make /free/I/U/N=(3,11) colors
	colors[0][]={{65535},{0},{0}} // red
	colors[1][]={{1},{26214},{0}} // dark green
	colors[2][]={{0},{0},{65535}} // blue
	colors[3][]={{26411},{1},{52428}} // purple
	colors[4][]={{0},{65535},{65535}} // light blue
	colors[5][]={{65535},{0},{52428}} // pink
	colors[6][]={{65535},{43690},{0}} // orange
	colors[7][]={{26214},{0},{0}} // maroon
	colors[8][]={{0},{65535},{0}} // green
	colors[9][]={{1},{3},{39321}} // dark blue
	colors[10][]={{1},{39321},{39321}} // teal
	colors[11][]={{39321},{1},{15729}} // crimson
	colors[12][]={{65535},{65535},{0}} // yellow
	colors[13][]={{0},{0},{0}} // black
	return colors
end

// hook function to create a tag as tke mouse cursor moves over points on the scatter plot preview
static function HookScatterPlot(STRUCT WMWinHookStruct &s)
	variable hookResult = 0
	
	switch(s.eventCode)
		case 0: // activated
			UpdateListboxes()
			break
		case 4: // mousemoved
			if (cmpstr(s.winName, "ScatterPanel#PreviewPlot")==0)
				string sHit = TraceFromPixel(s.mouseloc.h,s.mouseloc.v, "WINDOW:ScatterPanel#PreviewPlot;DELTAX:4;DELTAY:4;")
				if (strlen(sHit))
					string tname = StringByKey("TRACE", sHit)
					int pnt = NumberByKey("HITPOINT", sHit)
					wave ywave = TraceNameToWaveRef("ScatterPanel#PreviewPlot", tname)
					wave xwave = XWaveRefFromTrace("ScatterPanel#PreviewPlot", tname)
					sprintf sHit, "%s[%d]=%g\r%s[%d]=%g", NameOfWave(ywave), pnt, ywave[pnt], NameOfWave(xwave), pnt, xwave[pnt]
					Tag/C/N=idtag/Z=1/B=(65535,65534,49151)/G=(43690,43690,43690)/D={1,20}/A=LT/X=-2.00/Y=-2.00/L=0/W=ScatterPanel#PreviewPlot $tname, pnt,"\\K(0,0,0)\\Z10"+sHit
				else // not on point
					Tag/K/W=ScatterPanel#PreviewPlot/N=idtag
				endif
			endif
			break
	endswitch
	
	return hookResult
end